/*!
* \file PathStructures.h
* \brief Definition of main constants and structures used in the library
* \author CSTB
* \version 1.0
*/

#ifndef _NMPB08_PATHSTRUCT_INCLUDED_
#define _NMPB08_PATHSTRUCT_INCLUDED_

#include "pathdefNMPB.h"
#include <vector>
#include "math.h"
#include "stdio.h"
using namespace std;

// on Unix-like systems there is no way_ to hide local classes and to prevent them from 
// being replaced by other classes with similar names in other libraries. The use of 
// unusual names may help preventing name clashes.

#ifndef _WIN32
#define  ProfilePointNMPB   _Local_PROPAN8_ProfilePointNMPB_
#define  PropagationPath	_Local_PROPAN8_PropagationPath_
#endif

#define NULE 0 

/*!
 * \brief sound speed in the air 
 */
#define cSound ((double) 340) 
/*!
* \brief used value for Pi
*/
#define PI ((double) 3.14159265358979323)

// Computes the distance between two 2D points
double distance2D(Position2D const* position1, Position2D const* position2);

// Computes the distance between two 3D points
double distance3D(Position3D const* position1, Position3D const* position2);

// Computes the ground distance between 2 positions
double GroundDistance(Position3D const* position1, Position3D const* position2);

/*!
* \brief Profile point Structure
*/
struct ProfilePointNMPB
{
	/*!
	 * \brief 3D coordinates (x, y, z)
	 */
	Position3D  position3D;  
	/*!
	* \brief 2D coordinates (d : cumulated distance, z)	
	*/
	Position2D  position2D; 
	/*!
	* \brief impedance value before the point
	*/
	double		impedance;
	/*!
	* \brief To know if there is diffraction on this point
	*/
	bool		isDiff ; 	
	/*!
	* \brief The local height
	*/
	double 		height ;		
	/*!
	* \brief To know height of the point to [SR] (or another ray of the convex hull) : < 0 if the ray is under the point
	*/
	double		h_ray;	
	/*!
	* \brief Extension for path elements
	*/
	ExtensionNMPB* ext ;

	/*!
	 * \brief ProfilePointNMPB initialization
	 */
	ProfilePointNMPB()
	{
		ext = 0 ;
		/*
		new ExtensionNMPB;
		ext->type = ETNone_NMPB;
		ext->height = 0;
		ext->alphaArray = NULE;
		ext->cosTheta = 1;
		*/
		isDiff = 0;
		impedance = 0;
		h_ray = 0;
		height = 0 ;
		position3D.x = 0;
		position3D.y = 0;
		position3D.z = 0;
		position2D.d = 0;
		position2D.z = 0;
	} ;

	/*!
	* \brief ProfilePointNMPB destructor
	*/
	~ProfilePointNMPB(void)
	{
		delete ext;
	}

	/*!
	 * \brief Gets the 2D coordinates with the local height added to z
	 * 
	 * \returns the position2D coordinates with height
	 */
	Position2D position2D_withHeight(void)
	{
		Position2D posWithHeight ;
		posWithHeight.d = position2D.d;
		posWithHeight.z = position2D.z + height;
		return posWithHeight;
	}

	/*!
	* \brief Gets the 3D coordinates with the local height added to z
	* 
	* \returns the position3D coordinates with height
	*/
	Position3D position3D_withHeight(void)
	{
		Position3D posWithHeight ;
		posWithHeight.x = position3D.x;
		posWithHeight.y = position3D.y;
		posWithHeight.z = position3D.z + height;
		return posWithHeight;
	}
} ;

/*!
* \brief Structure for the propagation path
*/
struct PropagationPath
{
	/*!
	 * \brief the terrain items list
	 */
	vector<ProfilePointNMPB*> pathPoints;
	/*!
	* \brief distance between S and R
	*/
	double distSR;
	/*!
	* \brief the frequencies list
	*/
	vector<double> frequencies;
	/*!
	* \brief the calculated favorable attenuations
	*/
	double* favorableAttenuations; 
	/*!
	* \brief the calculated homogeneous attenuations
	*/
	double* homogeneousAttenuations; 

	/*!
	* \brief PropagationPath constructor
	*/
	PropagationPath(void)
	: pathPoints()
	, frequencies()
	{
		distSR = 0;
		_selectedOptions = 0;
		favorableAttenuations = 0 ; 
		homogeneousAttenuations = 0 ;
	} ;

	/*!
	* \brief PropagationPath destructor
	*/
	~PropagationPath(void)
	{
		ClearPath() ;
		delete [] favorableAttenuations;
		delete [] homogeneousAttenuations;
	}

	/*!
	 * \brief adds a new terrain item to the terrainItems list 
	 * 
	 * \param point3D The 3D coordinates
	 * 
	 * \param g The ground impedance before the point
	 */
	void ExtendPath (Position3D const* point3D, double g)
	{
		ProfilePointNMPB* pathPoint = new ProfilePointNMPB;
		pathPoint->position3D.x = point3D->x;
		pathPoint->position3D.y = point3D->y;
		pathPoint->position3D.z = point3D->z;
		pathPoint->impedance = g;
		/*
		ExtensionNMPB* ext = new ExtensionNMPB;
		ext->height = 0;
		ext->alphaArray = NULE;
		ext->type = ETNone_NMPB ;
		ext->cosTheta = 1;
		pathPoint->ext = ext;
		*/
		pathPoints.push_back(pathPoint);
	}

	/*!
	* \brief Adds a new terrain item to the terrainItems list, with extension data 
	* 
	* \param point3D The 3D coordinates
	* 
	* \param g The ground impedance before the point
	*
	* \param ext The extension data
	*/
	void ExtendPathExt (Position3D const* point3D, double g,  ExtensionNMPB const* ext)
	{
		if(ext != NULE && ext->type == ETScreen_NMPB)
		{
			// Screen to add to path : 3 points :
			ProfilePointNMPB* pathPoint1 = new ProfilePointNMPB;
			pathPoint1->position3D.x = point3D->x;
			pathPoint1->position3D.y = point3D->y;
			pathPoint1->position3D.z = point3D->z;
			pathPoint1->impedance = g;
			pathPoints.push_back(pathPoint1);

			ProfilePointNMPB* pathPoint2 = new ProfilePointNMPB;
			pathPoint2->position3D.x = point3D->x;
			pathPoint2->position3D.y = point3D->y;
			pathPoint2->position3D.z = point3D->z + ext->height;
			pathPoint2->impedance = g;
			pathPoints.push_back(pathPoint2);

			ProfilePointNMPB* pathPoint3 = new ProfilePointNMPB;
			pathPoint3->position3D.x = point3D->x + 0.001;
			pathPoint3->position3D.y = point3D->y;
			pathPoint3->position3D.z = point3D->z;
			pathPoint3->impedance = g;
			pathPoints.push_back(pathPoint3);
		}
		else
		{
			ProfilePointNMPB* pathPoint = new ProfilePointNMPB;
			pathPoint->position3D.x = point3D->x;
			pathPoint->position3D.y = point3D->y;
			pathPoint->position3D.z = point3D->z;
			pathPoint->impedance = g;
			if(ext != NULE)
			{
				pathPoint->ext = new ExtensionNMPB ;
				pathPoint->ext->type = ext->type;
				pathPoint->ext->height = ext->height;
				pathPoint->ext->alphaArray = ext->alphaArray;
				pathPoint->ext->cosTheta = ext->cosTheta;
				if(ext->type == ETSideDiffraction_NMPB)
				{
					// height used in propagation path for side diffractions but not for reflections (in this case, it's only used for retrodiffraction)
					pathPoint->height = ext->height; 
				}
			}

			pathPoints.push_back(pathPoint);
		}
	}

	/*!
	 * \brief Sets source height
	 * 
	 * \param h The source height
	 * 
	 * \returns true if all OK
	 * 
	 * \throws ERRNoPoint
	 */
	bool SetSourceHeight (double h) 
	{
		if (!pathPoints.empty())
		{
			pathPoints[0]->height = h;
			return true;
		}
		else
		{
			printf (".ERROR: path empty \n") ;
			throw new int( ERRNoPoint );
		}
		
		return false;
	}

	/*!
	* \brief Sets receiver height
	* 
	* \param h The receiver height
	* 
	* \returns true if all OK
	* 
	* \throws ERRNoPoint
	*/
	bool SetReceiverHeight (double h)
	{
		if (!pathPoints.empty())
		{
			pathPoints[pathPoints.size()-1]->height = h;
			return true;
		}
		else
		{
			printf (".ERROR: path empty \n") ;
			throw new int( ERRNoPoint );
		}

		return false;
	}

	/*!
	 * \brief Gets Source
	 * 
	 * \returns the source
	 */
	ProfilePointNMPB* GetSource()
	{
		if (!pathPoints.empty())
		{
			return pathPoints[0];
		}

		return 0;
	}

	/*!
	* \brief Gets Receiver
	* 
	* \returns the receiver
	*/
	ProfilePointNMPB* GetReceiver()
	{
		if (!pathPoints.empty())
		{
			return pathPoints[pathPoints.size()-1];
		}

		return 0;
	}

	/*!
	 * \brief Set frequencies
	 * 
	 * \param nbFreq The frequencies number
	 * 
	 * \param freq The frequencies array
	 */
	void SetFrequencies (int nbFreq, double const* freq)
	{
		frequencies.clear();
		if (nbFreq > 0)
		{
			for (int i = 0 ; i < nbFreq ; i++)
			{
				frequencies.push_back (freq[i]);
			}
		}
		favorableAttenuations   = new double[nbFreq];
		homogeneousAttenuations = new double[nbFreq];
	}

	/*!
	 * \brief Gets frequencies number
	 * 
	 * \returns the number of frequencies
	 */
	int GetNbFrequencies ()
	{
		return (int) frequencies.size() ;
	}

	/*!
	 * \brief Gets frequencies
	 * 
	 * \returns the frequencies array
	 */
	double const* GetFrequencies ()
	{
		int nbFreq = (int) frequencies.size();
		if (nbFreq > 0)
		{
			double const* freqArr = &frequencies[0];
			return freqArr;
		}

		return 0;
	}

	/*!
	 * \brief Search a frequency in the frequencies array
	 * 
	 * \param freq The searched frequency
	 * 
	 * \returns the frequency position in the array (-1 if not found)
	 */
	int GetFrequencyPosition(double freq)
	{
		int fPos = -1;
		int nbFreq = (int) frequencies.size();
		if (nbFreq > 0)
		{
			for(int i = 0 ; i < nbFreq ; i++)
			{
				if (frequencies[i] == freq) return i;
			}
		}

		return fPos;
	}

	/*!
	* \brief Set an option for the path
	* 
	* \param option The option to set
	* 
	* \param on_off True if the option must be set to true
	*/
	void SetOption(Option option, bool on_off)
	{
		if(on_off)
		{
			_selectedOptions |= option; 
		}
		else
		{
			_selectedOptions &= ~option; 
		}
	}

	/*!
	 * \brief Get the value of the option
	 * 
	 * \param option The option to check
	 * 
	 * \returns true if the option is selected
	 */
	bool GetOption(Option option)
	{
		return (option == (option & _selectedOptions));
	}

	/*!
	 * \brief Clears the path items list
	 */
	void ClearPath()
	{
		for (int i = 0 ; i < pathPoints.size() ; ++i) delete pathPoints[i] ;
		pathPoints.clear() ;
		distSR = 0;
	}

	private:
		/*!
		* \brief Indicates the selected options
		*/
		int _selectedOptions;
};

// Fills the TerrainItem Position2D
void FillPlanePosition(ProfilePointNMPB* terrain, ProfilePointNMPB const* source, double cumDistance);


#endif

