/*!
* \file Diffraction.cpp
* \brief Calculation of diffraction attenuations
* \author CSTB
* \version 1.0
*/

#include "Diffraction.h"
#include <math.h>
#include <stdio.h>
#include <assert.h>

namespace DiffractionNMPB
{

	// ------------------------------ Path difference -------------------------------------------------

	/*!
	* \brief Calculates the circular ray length for MN
	* 
	* \param distMN The MN distance
	* 
	* \param curvatureRadius The radius of curvature
	* 
	* \returns the circular ray length
	*/
	double CurveRayLength(double distMN, double curvatureRadius)
	{
		double curveLength = 0;
		if (curvatureRadius != 0)
		{
			curveLength = 2 * curvatureRadius * asin(distMN / (2 * curvatureRadius));
		}

		return curveLength;
	}

	/*!
	* \brief Calculation of the path difference for the screen elements
	*
	* p.47-49 -  9.4.3
	* 
	* \param source2D : source coordinates
	* 
	* \param receiver2D : receiver coordinates
	* 
	* \param screenItems : vector containing the terrain elements (terrain elements with diffracted indicated on the path, ie convex hull)
	* 
	* \param favourableConditions : true if calculation must be in favorable conditions, false for homogeneous conditions
	* 
	* \returns the path difference
	*/
	double PathDifference(Position2D const* source2D, Position2D const* receiver2D, vector<ProfilePointNMPB*> screenItems, bool favourableConditions)
	{
		double pathDifference;

		// gets all the kept screens :
		int usedScreensNumber = 0;
		vector<ProfilePointNMPB*> usedScreens;
		for (int i = 0; i < (int) screenItems.size(); i++)
		{
			ProfilePointNMPB* screenToCheck = screenItems[i];
			if (screenToCheck != 0 && screenToCheck->isDiff == 1)
			{
				usedScreens.push_back(screenToCheck);
				usedScreensNumber++;
			}
		}

		if (usedScreensNumber == 0)
		{
			return 0;
		}

		double distSR = distance2D(source2D, receiver2D);
		if (distSR == 0)
		{
			return 0;
		}

		if(favourableConditions)
		{
			// favorable conditions (p.47-49 -  9.4.3.2)
			double curvatureRadius = 8 * distSR; // p.47 -  9.4.3.2 - Formula (33)
			if (curvatureRadius < 1000)
			{
				curvatureRadius = 1000;
			}

			if (usedScreensNumber == 1)
			{
				// only one screen 
				ProfilePointNMPB* screen = usedScreens[0];
				Position2D posScreen = screen->position2D_withHeight();
				if (screen->h_ray <= 0)
				{
					
					pathDifference = CurveRayLength(distance2D(source2D, &posScreen), curvatureRadius) 
						           + CurveRayLength(distance2D(&posScreen, receiver2D), curvatureRadius) 
								   - CurveRayLength(distSR, curvatureRadius);
				}
				else
				{
					Position2D pointA;
					pointA.d = posScreen.d;
					pointA.z = posScreen.z + screen->h_ray;
					pathDifference = 2 * CurveRayLength(distance2D(source2D, &pointA), curvatureRadius) 
						           + 2 * CurveRayLength(distance2D(&pointA, receiver2D), curvatureRadius) 
								   - CurveRayLength(distance2D(source2D, &posScreen), curvatureRadius) 
								   - CurveRayLength(distance2D(&posScreen, receiver2D), curvatureRadius) 
								   - CurveRayLength(distSR, curvatureRadius);
				}
			}
			else
			{
				// multiple screens 
				double distScreens = 0;
				Position2D posScreen1, posScreen2;
				for (int i = 1; i < usedScreensNumber; i++)
				{
					posScreen1 = usedScreens[i-1]->position2D_withHeight();
					posScreen2 = usedScreens[i]->position2D_withHeight();
					distScreens += CurveRayLength(distance2D(&posScreen1,&posScreen2), curvatureRadius);					
				}
				posScreen1 = usedScreens[0]->position2D_withHeight();
				posScreen2 = usedScreens[usedScreensNumber-1]->position2D_withHeight();
				pathDifference = CurveRayLength(distance2D(source2D, &posScreen1), curvatureRadius) 
					           + distScreens 
							   + CurveRayLength(distance2D(&posScreen2, receiver2D), curvatureRadius) 
							   - CurveRayLength(distSR, curvatureRadius);
			}		
		}
		else
		{
			// homogeneous conditions (p.47 -  9.4.3.1)
			if (usedScreensNumber == 1)
			{
				// only one screen 
				ProfilePointNMPB* screen = usedScreens[0];
				Position2D posScreen = screen->position2D_withHeight();
				pathDifference = distance2D(source2D, &posScreen) 
					           + distance2D(&posScreen, receiver2D) 
							   - distSR;
				
				/*
				 * BUG CORRECTED
				 *
				 * DVM 28/11/2011
				 *
				 * screen->h_ray is valid only for real source to real receiver
				 *
				 * as we also use this calculation for image sources and/or receivers (e.g. when 
				 * estimating ground effects left and right of a diffracting edge), we must recalculate
				 * the height of the ray-path for every situation...
				 */

				double d1 = source2D->d ;
				double z1 = source2D->z ;
				double d2 = receiver2D->d ;
				double z2 = receiver2D->z ;
				double dd = posScreen.d ;
				double zd = posScreen.z ;
				double zr = z1 + (z2 - z1) * (dd - d1) / (d2 - d1) ;

				//if (screen->h_ray > 0)
				if (zr > zd)
				{
					pathDifference = -1 * pathDifference;					
				}
			}
			else
			{
				// multiple screens 
				double distScreens = 0;
				Position2D posScreen1, posScreen2;
				for (int i = 1; i < usedScreensNumber; i++)
				{
					posScreen1 = usedScreens[i-1]->position2D_withHeight();
					posScreen2 = usedScreens[i]->position2D_withHeight();
					distScreens += distance2D(&posScreen1,&posScreen2);
				}
				posScreen1 = usedScreens[0]->position2D_withHeight();
				posScreen2 = usedScreens[usedScreensNumber-1]->position2D_withHeight();
				pathDifference = distance2D(source2D, &posScreen1) 
					           + distScreens 
							   + distance2D(&posScreen2, receiver2D) 
							   - distSR;
			}		
		}

		return pathDifference;
	}

	/*!
	 * \brief Calculation of the path difference for a reflection element
	 *
	 * p.53 -  9.5.2 formula (44)
	 * 
	 * \param source2D : source coordinates
	 * 
	 * \param receiver2D : receiver coordinates
	 * 
	 * \param reflectionItem The reflection item
	 * 
	 * \returns the path difference
	 */
	double PathDifference(Position2D const* source2D, Position2D const* receiver2D, ProfilePointNMPB* reflectionItem)
	{
		double pathDifference;

		double distSR = distance2D(source2D, receiver2D);
		if (distSR == 0)
		{
			return 0;
		}

		reflectionItem->height = reflectionItem->ext->height;
		Position2D reflexPos = reflectionItem->position2D_withHeight();
		pathDifference = -1 * (distance2D(source2D, &reflexPos) + distance2D(&reflexPos, receiver2D) - distSR);

		return pathDifference;
	}

	/*!
	* \brief Calculation of the path difference for the side diffractions
	*
	* p.35-36 -  8.2.3 and p.47-49 -  9.4.3 and p.51 -  9.4.5
	* 
	* \param source3D : source coordinates
	* 
	* \param receiver3D : receiver coordinates
	* 
	* \param screenItems : vector containing the terrain elements 
	* 
	* \param totalDiffDist : distance between the first and the last diffraction (calculated in this function)
	* 
	* \returns the path difference
	*/
	double SidePathDifference(Position3D const* source3D, Position3D const* receiver3D, vector<ProfilePointNMPB*> screenItems, double &totalDiffDist)
	{
		double pathDifference = 0;

		// gets all the kept screens : 
		// 1 or 2 edges and the last reflection screen before edges and the first reflection screen after edges
		int usedEdgesNumber = 0;
		int usedRefNumber = 0;
		vector<ProfilePointNMPB*> usedEdges;
		vector<ProfilePointNMPB*> usedReflections;
		int posFirstEdge = 0;
		int posLastEdge = 0;
		int posLastRefBefore = 0;
		int posFirstRefAfter = 0;
		for (int i = 0; i < (int) screenItems.size(); i++)
		{
			ProfilePointNMPB* screenToCheck = screenItems[i];
			if (screenToCheck != 0 && screenToCheck->ext != 0)
			{
				if(screenToCheck->ext->type == ETSideDiffraction_NMPB)
				{
					if(posFirstEdge == 0)
					{
						posFirstEdge = i;
					}

					posLastEdge = i;

					usedEdges.push_back(screenToCheck);
					usedEdgesNumber++;
				}
				else if(screenToCheck->ext->type == ETReflection_NMPB)
				{
					if(posFirstEdge == 0)
					{
						posLastRefBefore = i;
						usedReflections.push_back(screenToCheck);
						usedRefNumber++;
					}
					else if(posLastEdge != 0 && posFirstRefAfter == 0)
					{
						posFirstRefAfter = i;
						usedReflections.push_back(screenToCheck);
						usedRefNumber++;
					}
				}
			}
		}

		if (usedEdgesNumber == 0)
		{
			return 0;
		}
		
		// SR or S'R' calculation (in reflection case) 
		// All the calculations are made in horizontal plane, so z = 0
		double distSR = 0;
		Position3D sourceImage;
		Position3D receiverImage;
		sourceImage.x = source3D->x;
		sourceImage.y = source3D->y;
		sourceImage.z = 0;
		receiverImage.x = receiver3D->x;
		receiverImage.y = receiver3D->y;
		receiverImage.z = 0;
		if(usedRefNumber > 0)
		{
			if (posLastRefBefore != 0 && posLastRefBefore < posFirstEdge)
			{
				ProfilePointNMPB* firstEdge = usedEdges[0];
				ProfilePointNMPB* refBefore = usedReflections[0];
				double mult = (firstEdge->position2D.d - screenItems[0]->position2D.d) / (firstEdge->position2D.d - refBefore->position2D.d);
				sourceImage.x = firstEdge->position3D.x  + (refBefore->position3D.x - firstEdge->position3D.x) * mult;
				sourceImage.y = firstEdge->position3D.y  + (refBefore->position3D.y - firstEdge->position3D.y) * mult;		
			}

			if (posFirstRefAfter != 0 && posFirstRefAfter > posLastEdge)
			{
				ProfilePointNMPB* lastEdge = usedEdges[usedEdgesNumber - 1];
				ProfilePointNMPB* refAfter = usedReflections[usedRefNumber - 1];
				double mult = (lastEdge->position2D.d - screenItems[screenItems.size() - 1]->position2D.d) / (lastEdge->position2D.d - refAfter->position2D.d);
				receiverImage.x = lastEdge->position3D.x  + (refAfter->position3D.x - lastEdge->position3D.x) * mult;
				receiverImage.y = lastEdge->position3D.y  + (refAfter->position3D.y - lastEdge->position3D.y) * mult;				
			}
		}

		distSR = GroundDistance(&sourceImage, &receiverImage);

		if (distSR == 0)
		{
			return 0;
		}

		// Path difference in horizontal plane (only homogeneous conditions)
		if (usedEdgesNumber == 1)
		{
			// only one edge 
			pathDifference = GroundDistance(&sourceImage, &usedEdges[0]->position3D) 
				           + GroundDistance(&usedEdges[0]->position3D, &receiverImage) - distSR;
			totalDiffDist = 0;
		}
		else
		{
			// multiple screens 
			totalDiffDist = usedEdges[usedEdgesNumber-1]->position2D.d - usedEdges[0]->position2D.d;
			pathDifference = GroundDistance(&sourceImage, &usedEdges[0]->position3D) 
				           + totalDiffDist 
						   + GroundDistance(&usedEdges[usedEdgesNumber-1]->position3D, &receiverImage) - distSR;
		}		 

		return pathDifference;
	}


	// ------------------------------ Diffraction Class -----------------------------------------

	/*!
	* \brief Initialization of the Diffraction Class
	* 
	* section 9.4 p.44-50
	* 
	* \param terrainItems The terrain items
	* 
	* \param freq Center frequency
	* 
	* \param freqPos The frequency position in the spectrum array
	* 
	* \param withEmbankment True if embankment must be checked
	* 
	* Initialization of the Diffraction Class with source, receiver, screens and frequency (containing mean planes calculation, SR path difference calculation and reflection attenuation)
	* 
	* \throws ERRFrequency 
	*/
	Diffraction::Diffraction(vector<ProfilePointNMPB*> terrainItems, double freq, int freqPos, bool withEmbankment)
	{
		if (terrainItems.size() < 2)
		{
			return;
		}

		_source = terrainItems[0];
		_receiver = terrainItems[terrainItems.size() - 1];
		_terrainItems = terrainItems;
		_freq = freq;

		if (freq != 0)
		{
			_wavelength =  cSound / freq;
		}
		else
		{
			printf (".ERROR: frequency = 0 \n") ;
			throw new int( ERRFrequency );
		}

		ClearData();

		// reflections attenuation
		ReflectionAttenuation(terrainItems, freqPos);

		// path differences
		Position2D posSource = _source->position2D_withHeight();
		Position2D posRecep = _receiver->position2D_withHeight();
		_pathDifferenceSR_h = PathDifference(&posSource, &posRecep, _terrainItems, false);
		_pathDifferenceSR_f = PathDifference(&posSource, &posRecep, _terrainItems, true);

		MeanPlanesDataCalculation(withEmbankment);
	}

	/*!
	 * \brief Initialization of the Diffraction Class in side diffraction case 
	 * 
	 * section 9.4.5 p.51
	 * 
	 * \param terrainItems the terrain items
	 * 
	 * \param freq center frequency
	 * 
	 * \param freqPos The frequency position in the spectrum array
	 * 
	 * \param nbSideDiffractions the side diffractions number
	 * 
	 * Initialization of the Diffraction Class in side diffraction case 
	 *	with source, receiver, screens and frequency (containing SR path difference calculation, reflection attenuation and _DeltaDifSR_h calculation)
	 * 
	 * \throws ERRFrequency
	 */
	Diffraction::Diffraction(vector<ProfilePointNMPB*> terrainItems, double freq, int freqPos, int nbSideDiffractions)
	{
		if (terrainItems.size() < 2)
		{
			return;
		}

		_source = terrainItems[0];
		_receiver = terrainItems[terrainItems.size() - 1];
		_terrainItems = terrainItems;
		_freq = freq;

		if (freq != 0)
		{
			_wavelength =  cSound / freq;
		}
		else
		{
			printf (".ERROR: frequency = 0 \n") ;
			throw new int( ERRFrequency );
		}

		ClearData();

		// reflections attenuation
		ReflectionAttenuation(terrainItems, freqPos);

		// side diffraction
		Position3D terrainPos1, terrainPos2;
		Position2D sourcePos, recepPos;
		terrainPos1 = terrainItems[0]->position3D_withHeight();
		terrainPos2 = terrainItems[(int)terrainItems.size()-1]->position3D_withHeight();
		sourcePos = _source->position2D_withHeight();
		recepPos = _receiver->position2D_withHeight();
		_pathDifferenceSR_h = SidePathDifference(&terrainPos1, &terrainPos2, terrainItems, _totalDiffDist);
		_DeltaDifSR_h = CalculDiffractionPure(&sourcePos , &recepPos, _terrainItems, false, true, false);
	}

	/*!
	 * \brief Calculates the reflection attenuation
	 * 
	 * \param terrainItems The terrain items
	 * 
	 * \param freqPos The frequency position in the spectrum array
	 */
	void Diffraction::ReflectionAttenuation(vector<ProfilePointNMPB*> &terrainItems, int freqPos) 
	{
		_absorptionAttenuation = 0;
		for (int i = 0 ; i < (int)terrainItems.size() ; i++)
		{
			ProfilePointNMPB* pointNMPB = terrainItems[i];
			if(pointNMPB->ext && pointNMPB->ext->type == ETReflection_NMPB)
			{
				_absorptionAttenuation += SoundAbsorption(pointNMPB, i, freqPos);
			}
		}
		// * (-1) because value taken as attenuation 
		_absorptionAttenuation = -1. * _absorptionAttenuation;
	}

	/*!
	* \brief Calculates the sound absorption due to reflection
	* 
	* \param pointNMPB The reflection screen
	* 
	* \param posRef The reflection position in the elements vector
	* 
	* \param freqPos The frequency position in the spectrum array
	* 
	* \returns the calculated absorption 
	* 
	* \throws ERRScreenAbsorption
	*/
	double Diffraction::SoundAbsorption(ProfilePointNMPB* pointNMPB, int posRef, int freqPos)
	{
		// absorption p.51 -  9.5.1 Formula (43)
		double* alphaArray = pointNMPB->ext->alphaArray;
		if(freqPos >= 0 && alphaArray != NULL)
		{
			double absorpCoeff = alphaArray[freqPos];

			double screenAbsorption = 0;
			if (absorpCoeff < 1)
			{
				screenAbsorption =  10 * log10(1-absorpCoeff);
			}
			else
			{
				printf (".ERROR: screen absorption %4.2f > 1 \n", screenAbsorption) ;
				throw new int( ERRScreenAbsorption );
			}

			// retro diffraction p.52-53-54 -  9.5.2
			double retroDiff = 0;
			if (pointNMPB->ext->height != 0)
			{
				pointNMPB->height = pointNMPB->ext->height;

				// retro diffraction between S and R, 
				//	or between E1 and E2 if E1  last diffraction screen between S and the reflection screen
				//							E2 first diffraction screen between the reflection screen and R
				ProfilePointNMPB* diffPointBefore = _source;
				for (int i = 1; i < posRef; i++)
				{
					ProfilePointNMPB* screenToCheck = _terrainItems[i];
					if (screenToCheck != 0 && screenToCheck->isDiff == 1 && screenToCheck->h_ray <= 0)
					{
						diffPointBefore = screenToCheck;
					}
				}

				ProfilePointNMPB* diffPointAfter = _receiver;
				for (int i = posRef + 1; i < (int) _terrainItems.size(); i++)
				{
					ProfilePointNMPB* screenToCheck = _terrainItems[i];
					if (screenToCheck != 0 && screenToCheck->isDiff == 1 && screenToCheck->h_ray <= 0)
					{
						diffPointAfter = screenToCheck;
						break;
					}
				}

				Position2D posBefore, posAfter;
				posBefore = diffPointBefore->position2D_withHeight();
				posAfter = diffPointAfter->position2D_withHeight();
				retroDiff = CalculDiffractionPure(&posBefore, &posAfter, pointNMPB);
			}

			//p.53 -  9.5.2 Formula (46)
			return screenAbsorption - retroDiff;
		}

		return 0;
	}

	/*!
	* \brief Calculates data of the 2 mean planes (SO) and (OR), and embankment
	*/
	void Diffraction::MeanPlanesDataCalculation(bool withEmbankment)
	{
		// search first and last used screens :
		_firstScreen = 0;
		_lastScreen = 0;
		int screenNumber = 0;
		int firstScreenPos = 0;
		int lastScreenPos = 0;
		for (int i = 0; i < (int) _terrainItems.size(); i++)
		{
			ProfilePointNMPB* screenToCheck = _terrainItems[i];
			if (screenToCheck != 0 && screenToCheck->isDiff == 1)
			{
				screenNumber++;
				if (_firstScreen == 0)
				{
					_firstScreen = screenToCheck;
					firstScreenPos = i;
				}

				_lastScreen = screenToCheck;
				lastScreenPos = i;
			}
		}
		
		if (screenNumber == 0)
		{
			_Adif_h = 0;
			_Adif_f = 0;
			return;
		}

		_totalDiffDist = 0;
		if (screenNumber > 1)
		{
			Position2D firstPos, lastPos;
			firstPos = _firstScreen->position2D_withHeight();
			lastPos = _lastScreen->position2D_withHeight();
			_totalDiffDist = distance2D(&firstPos , &lastPos);
		}

		// Mean planes (p.45 figure 21)
		vector<ProfilePointNMPB*> terrainsS;
		vector<ProfilePointNMPB*> terrainsR;
		for (int i = 0 ; i < (int) _terrainItems.size(); i++ )
		{
			ProfilePointNMPB* terrain = _terrainItems[i];
			if (i <= firstScreenPos)
			{
				terrainsS.push_back(terrain);
			}

			if (i >= lastScreenPos)
			{
				terrainsR.push_back(terrain);
			}
		}

		MeanPlane meanPlaneSO(terrainsS);
		_meanPlaneSO = meanPlaneSO;
		MeanPlane meanPlaneOR(terrainsR);
		_meanPlaneOR = meanPlaneOR;

		_h0 = max(meanPlaneSO.Get_zEqReceiver(), meanPlaneOR.Get_zEqSource());	

		// Embankment calculations
		_AttTalusSO = 0;
		if(withEmbankment)
		{
			Embankment embankment(terrainsS, _freq);
			_AttTalusSO = embankment.GetEmbankmentAttenuation();
			_embankmentSourceImage.d = embankment.GetSourceImage()->d;
			_embankmentSourceImage.z = embankment.GetSourceImage()->z;
		}
	}

	/*!
	 * \brief Calculation of the pure diffraction
	 * 
 	 *  p.46 -  9.4.2 Formula (31)
 	 * 
	 * \param source2D The source coordinates
	 * 
	 * \param receiver2D The receiver coordinates
	 * 
	 * \param screenItems The screen items
	 * 
	 * \param favourableConditions : true if conditions are favorable (false if homogeneous)
	 * 
	 * \param totalPath : true if the calculation is on the total path
	 *
	 * \param withCh : true if Ch must be calculated (false if Ch = 1)
	 *
	 * \returns the calculated pure diffraction
	 */
	double Diffraction::CalculDiffractionPure (Position2D const* source2D, 
											   Position2D const* receiver2D, 
											   vector<ProfilePointNMPB*> screenItems, 
											   bool favourableConditions, 
											   bool totalPath, 
											   bool withCh)
	{
		// calculation of coefficient for multiple diffractions (C'')
		// Formula (32)
		double multiDiffCoef = 1;
		if (_totalDiffDist >= 0.3)
		{
			double val = pow(5. * _wavelength / _totalDiffDist, 2);
			multiDiffCoef = (1. + val) / (1./3. + val);
		}

		// calculation of Ch coefficient
		double Ch = 1;
		if(withCh)
		{
			double Chtemp = _freq * _h0 / 250.0;
			if (Chtemp < 1)
			{
				Ch = Chtemp;
			}
		}

		// path difference calculation
		double pathDifference;
		if (totalPath)
		{
			// SR path already calculated
			if (favourableConditions)
			{
				pathDifference = _pathDifferenceSR_f;
			}
			else
			{
				pathDifference = _pathDifferenceSR_h;
			}
		}
		else
		{
			pathDifference = PathDifference(source2D, receiver2D, screenItems, favourableConditions);
		}


		// diffraction calculation 
		double deltaDif = 0;
		double comparisonVal = 40 * multiDiffCoef * pathDifference / _wavelength;
		if (comparisonVal >= -2)
		{
			deltaDif = 10 * Ch * log10(3. + comparisonVal);
			if (deltaDif < 0) deltaDif = 0;
		}
	
		return deltaDif;
	}

	/*!
	* \brief Calculation of the retro diffraction in reflection case
	* 
	*  p.53 -  9.5.2 Formula (45)
	* 
	* \param source2D The source coordinates
	* 
	* \param receiver2D The receiver coordinates
	 * 
	 * \param reflectionItem The vertical reflection screen
	 * 
	 * \returns the calculated retro diffraction
	 */
	double Diffraction::CalculDiffractionPure (Position2D const* source2D, 
											   Position2D const* receiver2D, 
											   ProfilePointNMPB* reflectionItem)
	{
		double pathDifference =  PathDifference(source2D, receiver2D, reflectionItem);

		// diffraction calculation 
		double deltaDif = 0;
		if (_wavelength != 0)
		{
			double comparisonVal = 40 * pathDifference / _wavelength;
			if (comparisonVal >= -2)
			{
				deltaDif = 10 * log10(3. + comparisonVal);
			}
		}

		return deltaDif;
	}

	/*!
	 * \brief Calculation of the ground attenuation with diffraction, source side or receiver side
	 * 
	 * \param aSol : ground attenuation (source side or receiver side)
	 * 
	 * \param deltaDif1 : diffraction attenuation during the image path (S'R or SR')
	 * 
	 * \param deltaDif2 : diffraction attenuation during the real path (SR)
	 * 
	 * \returns
	 */
	double Diffraction::CalculDeltaSol(double aSol, double deltaDif1, double deltaDif2)
	{
		/*
		 * BUG CORRECTION
		 *
		 * DVM 28/11/2011
		 *
		 * weighting of ground effect by means of diffraction may become undefined if the 
		 * image source (or receiver) is less attenuated than the real source (or receiver) ;
		 * this could happen if the source or receiver was found below the mean plane for
		 * the ground segments on either side of a diffracting edge
		 *
		 * assert added in case this happens... no solution proposed
		 */
		assert (deltaDif1 >= deltaDif2) ;

		double deltaSol = -20 * log10(1 + (pow(10, -1 * aSol / 20.0) - 1) * pow(10, -1 * (deltaDif1 - deltaDif2) / 20.0));
		return deltaSol;
	}

	/*!
	 * \brief Calculates attenuation due to diffraction 
	 * 
	 * Calculates attenuation due to diffraction, with ground effects and embankment
	 * p.49 -  9.4.4 formula (37)
	 * 
	 */
	void Diffraction::CalculAttenuationDiffraction (bool withCh)
	{
		// Checks path difference :
		double minPathDifference = -1 * _wavelength / 20;
		bool doHomogeneous = _pathDifferenceSR_h >= minPathDifference;
		bool doFavorable = _pathDifferenceSR_f >= minPathDifference;

		if (!doHomogeneous && !doFavorable)
		{
			_Adif_h = 0;
			_Adif_f = 0;
			return;
		}

		// intermediate calculations for ground
		GroundEffect grEffSO(_meanPlaneSO.Get_dp(),_meanPlaneSO.Get_zEqSource(), _meanPlaneSO.Get_zEqReceiver(), _meanPlaneSO.Get_Gpath(), _meanPlaneSO.Get_Gsource(), _freq, DeltaSol_SO);
		GroundEffect grEffOR(_meanPlaneOR.Get_dp(),_meanPlaneOR.Get_zEqSource(), _meanPlaneOR.Get_zEqReceiver(), _meanPlaneOR.Get_Gpath(), _meanPlaneOR.Get_Gsource(), _freq, DeltaSol_OR);

		Position2D posSource, posRecep;
		posSource = _source->position2D_withHeight();
		posRecep = _receiver->position2D_withHeight();
		// homogeneous case
		if(doHomogeneous)
		{
			// calculation of pure diffraction between source and receiver :
			_DeltaDifSR_h = CalculDiffractionPure(&posSource, &posRecep, _terrainItems, false, true, withCh);

			// calculation of ground attenuation with diffraction, source side: formula (38)
			_aSolSO_h = grEffSO.AttenuationCalculationH();
			Position2D sourceImage = _meanPlaneSO.Get_sourceImage();
			double deltaDifSO_h = CalculDiffractionPure(&sourceImage, &posRecep, _terrainItems, false, false, withCh);
			_DeltaSolSO_h = CalculDeltaSol(_aSolSO_h, deltaDifSO_h, _DeltaDifSR_h);
			
			// calculation of ground attenuation with diffraction, receiver side: formula (40)
			double aSolOR_h = grEffOR.AttenuationCalculationH();
			Position2D recepImage = _meanPlaneOR.Get_receiverImage();
			double deltaDifOR_h = CalculDiffractionPure(&posSource, &recepImage, _terrainItems, false, false, withCh);
			_DeltaSolOR_h = CalculDeltaSol(aSolOR_h, deltaDifOR_h, _DeltaDifSR_h);
			
			// calculation of embankment source side attenuation : formula (39)
			_DeltaTalusSO_h = 0;
			if (_AttTalusSO != 0)
			{
				double deltaDifTalusSO_h = CalculDiffractionPure (&_embankmentSourceImage, &posRecep, _terrainItems, false, false, withCh);
				_DeltaTalusSO_h = CalculDeltaSol(_AttTalusSO, deltaDifTalusSO_h, _DeltaDifSR_h);
			}

			// calculation of the total diffraction attenuation : formula (37)
			if (_DeltaDifSR_h > 25)
			{
				// p.46 -  9.4.2 
				_DeltaDifSR_h = 25;
			}

			_Adif_h = _DeltaDifSR_h + _DeltaSolSO_h + _DeltaTalusSO_h + _DeltaSolOR_h;
		}
		else
		{
			_Adif_h = 0;
		}

		// favorable case
		if(doFavorable)
		{
			// calculation of pure diffraction between source and receiver :
			_DeltaDifSR_f = CalculDiffractionPure(&posSource, &posRecep, _terrainItems, true, true, withCh);

			// calculation of ground attenuation with diffraction, source side : formula (38)
			_aSolSO_f = grEffSO.AttenuationCalculationF();
			Position2D sourceImage = _meanPlaneSO.Get_sourceImage();
			double deltaDifSO_f = CalculDiffractionPure(&sourceImage, &posRecep, _terrainItems, true, false, withCh);
			_DeltaSolSO_f = CalculDeltaSol(_aSolSO_f, deltaDifSO_f, _DeltaDifSR_f);

			// calculation of ground attenuation with diffraction, receiver side: formula (40)
			double aSolOR_f = grEffOR.AttenuationCalculationF();
			Position2D recepImage = _meanPlaneOR.Get_receiverImage();
			double deltaDifOR_f = CalculDiffractionPure(&posSource, &recepImage, _terrainItems, true, false, withCh);
			_DeltaSolOR_f = CalculDeltaSol(aSolOR_f, deltaDifOR_f, _DeltaDifSR_f);

			// calculation of embankment source side attenuation : formula (39)
			_DeltaTalusSO_f = 0;
			if (_AttTalusSO != 0)
			{
				double deltaDifTalusSO_f = CalculDiffractionPure(&_embankmentSourceImage, &posRecep, _terrainItems, true, false, withCh);
				_DeltaTalusSO_f = CalculDeltaSol(_AttTalusSO, deltaDifTalusSO_f, _DeltaDifSR_f);
			}

			// calculation of the total diffraction attenuation : formula (37)
			if (_DeltaDifSR_f > 25)
			{
				// p.46 -  9.4.2 
				_DeltaDifSR_f = 25;
			}

			_Adif_f = _DeltaDifSR_f + _DeltaSolSO_f + _DeltaTalusSO_f + _DeltaSolOR_f;
		}
		else
		{
			_Adif_f = 0;
		}		
	}
}
