/*!
* \file Diffraction.h
* \brief Calculation of diffraction attenuations
* \author CSTB
* \version 1.0
*/

#ifndef DIFFRACTION_H
#define DIFFRACTION_H

#include "GroundEffect.h"
#include "Embankment.h"

#ifndef _WIN32
#define  Diffraction   _Local_PROPAN8_Diffraction_
#endif

using namespace GroundEffectNMPB;
using namespace EmbankmentNMPB;

namespace DiffractionNMPB
{
    /*!
	 * \brief Class used to calculate diffraction attenuation
	 * 
	 * Class used to calculate diffraction attenuation with intermediates values and functions ;
	 * Contains ground attenuation ; uses PathDifference
	 * p. 44-51 -  9.4
	 * 
	 */
	class Diffraction
    {
    public:

		/*!
		* \brief Initialization of the Diffraction Class
		*/
		Diffraction()
		{
			_freq = 0;
			_wavelength = 0;
			ClearData();
		}

		/*!
		* \brief Initialization of the Diffraction Class
		* 
		* \param terrainItems The terrain items
		* 
		* \param freq Center frequency
		* 
		* \param freqPos The frequency position in the spectrum array
		* 
		* \param withEmbankment True if embankment must be checked
		* 
		* Initialization of the Diffraction Class with source, receiver, screens and frequency (containing mean planes calculation, SR path difference calculation and reflection attenuation)
		* 
		* \throws ERRFrequency 
		*/
		Diffraction(vector<ProfilePointNMPB*> terrainItems, double freq, int freqPos, bool withEmbankment);

		/*!
		* \brief Initialization of the Diffraction Class in side diffraction case 
		* 
		* \param terrainItems the terrain items
		* 
		* \param freq center frequency
		* 
		* \param freqPos The frequency position in the spectrum array
		* 
		* \param nbSideDiffractions the side diffractions number
		* 
		* Initialization of the Diffraction Class in side diffraction case 
		*	with source, receiver, screens and frequency (containing SR path difference calculation, reflection attenuation and _DeltaDifSR_h calculation)
		* 
		* \throws ERRFrequency
		*/
		Diffraction(vector<ProfilePointNMPB*> terrainItems, double freq, int freqPos, int nbSideDiffractions);

		/*!
		* \brief Calculates the reflection attenuation
		* 
		* \param terrainItems The terrain items
		* 
		* \param freqPos The frequency position in the spectrum array
		*/
		void ReflectionAttenuation(vector<ProfilePointNMPB*> &terrainItems, int freqPos);

		/*!
		* \brief Calculates attenuation due to diffraction 
		* 
		* Calculates attenuation due to diffraction, with ground effects and embankment
		* p.49 -  9.4.4 formula (37)
		* 
		* \param withCh if true, enables the calculation Ch, the correction term for low height obstacles, 
		*               if false, Ch is taken equal to 1 
		*/
		void CalculAttenuationDiffraction (bool withCh = true);

		/*!
		* \brief Calculation of the pure diffraction
		* 
		*  p.46 -  9.4.2 Formula (31)
		* 
		* \param source2D The source coordinates
		* 
		* \param receiver2D The receiver coordinates
		* 
		* \param screenItems The screen items
		* 
		* \param favourableConditions : true if conditions are favorable (false if homogeneous)
		* 
		* \param totalPath : true if the calculation is on the total path
		*
		* \param withCh : true if Ch must be calculated (false if Ch = 1)
		*
		* \returns the calculated pure diffraction
		*/
		double CalculDiffractionPure (Position2D const* source2D, 
			                          Position2D const* receiver2D, 
									  vector<ProfilePointNMPB*> terrainItems, 
									  bool favourableConditions, 
									  bool totalPath, 
									  bool withCh);
		
		/*!
		* \brief Calculation of the retro diffraction in reflection case
		* 
		*  p.53 -  9.5.2 Formula (45)
		* 
		* \param source2D The source coordinates
		* 
		* \param receiver2D The receiver coordinates
		* 
		* \param reflectionItem The vertical reflection screen
		* 
		* \returns the calculated retro diffraction
		*/
		double CalculDiffractionPure (Position2D const* source2D, 
			                          Position2D const* receiver2D, 
									  ProfilePointNMPB* reflectionItem);

		/*!
		 * \brief Gets path difference for the SR path, in homogeneous conditions (used to know if diffraction must be calculated)
		 * 
		 * \returns the path difference for the SR path in homogeneous conditions
		 */
		double Get_pathDifferenceSR_h()
		{
			return _pathDifferenceSR_h;
		}

		/*!
		* \brief Gets path difference for the SR path, in favorable conditions (used to know if diffraction must be calculated)
		* 
		* \returns the path difference for the SR path in favorable conditions
		*/
		double Get_pathDifferenceSR_f()
		{
			return _pathDifferenceSR_f;
		}

		/*!
		* \brief Gets diffraction attenuation (Adif), in homogeneous conditions
		* 
		* \returns diffraction attenuation (Adif), in homogeneous conditions
		*/
		double Get_diffractionAttenuation_h()
		{
			return _Adif_h;
		}

		/*!
		* \brief Gets diffraction attenuation (Adif), in favorable conditions
		* 
		* \returns diffraction attenuation (Adif), in favorable conditions
		*/
		double Get_diffractionAttenuation_f()
		{
			return _Adif_f;
		}

		/*!
		* \brief Gets attenuation due to pure diffraction, in homogeneous conditions
		* 
		* \returns attenuation due to pure diffraction, in homogeneous conditions
		*/
		double Get_DeltaDifSR_h()
		{
			return _DeltaDifSR_h;
		}

		/*!
		* \brief Gets attenuation due to pure diffraction, in favorable conditions
		* 
		* \returns attenuation due to pure diffraction, in favorable conditions
		*/
		double Get_DeltaDifSR_f()
		{
			return _DeltaDifSR_f;
		}

		/*!
		* \brief Gets attenuation from the ground effect source side, weighted by the diffraction source side, in homogeneous conditions
		* 
		* \returns attenuation from the ground effect source side, weighted by the diffraction source side, in homogeneous conditions
		*/
		double Get_DeltaSolSO_h()
		{
			return _DeltaSolSO_h;
		}

		/*!
		* \brief Gets attenuation from the ground effect source side, weighted by the diffraction source side, in favorable conditions
		* 
		* \returns attenuation from the ground effect source side, weighted by the diffraction source side, in favorable conditions
		*/
		double Get_DeltaSolSO_f()
		{
			return _DeltaSolSO_f;
		}

		/*!
		* \brief Gets attenuation from the ground effect receiver side, weighted by the diffraction receiver side, in homogeneous conditions
		* 
		* \returns attenuation from the ground effect receiver side, weighted by the diffraction receiver side, in homogeneous conditions
		*/
		double Get_DeltaSolOR_h()
		{
			return _DeltaSolOR_h;
		}

		/*!
		* \brief Gets attenuation from the ground effect receiver side, weighted by the diffraction receiver side, in favorable conditions
		* 
		* \returns attenuation from the ground effect receiver side, weighted by the diffraction receiver side, in favorable conditions
		*/
		double Get_DeltaSolOR_f()
		{
			return _DeltaSolOR_f;
		}

		/*!
		* \brief Gets attenuation due to embankment in homogeneous conditions
		* 
		* \returns attenuation due to embankment for the SO path in homogeneous conditions
		*/
		double Get_DeltaTalusSO_h()
		{
			return _DeltaTalusSO_h;
		}

		/*!
		* \brief Gets attenuation due to embankment in favorable conditions
		* 
		* \returns attenuation due to embankment for the SO path in favorable conditions
		*/
		double Get_DeltaTalusSO_f()
		{
			return _DeltaTalusSO_f;
		}

		/*!
		* \brief Gets attenuation due to the ground effect for the SO path in homogeneous conditions
		* 
		* \returns attenuation due to the ground effect for the SO path in homogeneous conditions
		*/
		double Get_AttsolSO_h()
		{
			return _aSolSO_h;
		}

		/*!
		* \brief Gets attenuation due to the ground effect for the SO path in favorable conditions
		* 
		* \returns attenuation due to the ground effect for the SO path in favorable conditions
		*/
		double Get_AttsolSO_f()
		{
			return _aSolSO_f;
		}

		/*!
		* \brief Gets attenuation due reflection absorption
		*
		* \returns attenuation due to reflection absorption
		*/
		double Get_absorptionAttenuation()
		{
			return _absorptionAttenuation;
		}

	protected:
		/*!
		 * \brief Source 
		 */
		ProfilePointNMPB* _source; 
		/*!
		* \brief Receiver
		*/
		ProfilePointNMPB* _receiver; 
		/*!
		* \brief Terrain items
		*/ 
		vector<ProfilePointNMPB*> _terrainItems;
		/*!
		* \brief Median frequency fm
		*/ 
		double _freq;	

		/*!
		* \brief the first used screen after source
		*/ 
		ProfilePointNMPB* _firstScreen;
		/*!
		* \brief the last used screen before receiver
		*/ 
		ProfilePointNMPB* _lastScreen;

		/*!
		* \brief the mean plane between source and first screen
		*/ 
		MeanPlane _meanPlaneSO;
		/*!
		* \brief the mean plane between last screen and receiver
		*/ 
		MeanPlane _meanPlaneOR;

		/*!
		* \brief wavelength (lambda)
		*/ 
		double _wavelength; 
		/*!
		* \brief total distance between diffraction closest to the source and diffraction closest to the receptor (e)
		*/ 
		double _totalDiffDist;
		/*!
		* \brief higher diffraction edge for the 2 mean planes (source side and receiver side)
		*/ 
		double _h0;

		/*!
		* \brief path differences for the SR path (in homogeneous conditions)
		*/ 
		double _pathDifferenceSR_h;
		/*!
		* \brief path differences for the SR path (in favorable conditions)
		*/ 
		double _pathDifferenceSR_f;
		/*!
		* \brief diffraction attenuation (Adif), in homogeneous conditions
		*/ 
		double _Adif_h;
		/*!
		* \brief diffraction attenuation (Adif), in favorable conditions
		*/ 
		double _Adif_f;
		/*!
		* \brief attenuation due to pure diffraction between source S et receiver R, in homogeneous conditions
		*/ 
		double _DeltaDifSR_h;
		/*!
		* \brief attenuation due to pure diffraction between source S et receiver R, in favorable conditions
		*/ 
		double _DeltaDifSR_f; 
		/*!
		* \brief attenuation due to source ground effect, in homogeneous conditions
		*/ 
		double _DeltaSolSO_h;
		/*!
		* \brief attenuation due to source ground effect, in favorable conditions
		*/ 
		double _DeltaSolSO_f;
		/*!
		* \brief attenuation due to embankment in source side
		*/ 
		double _AttTalusSO;
		/*!
		* \brief the embankment source image position
		*/ 
		Position2D _embankmentSourceImage;
		/*!
		* \brief attenuation due to embankment in source side, in homogeneous conditions
		*/ 
		double _DeltaTalusSO_h;
		/*!
		* \brief attenuation due to embankment in source side, in favorable conditions
		*/ 
		double _DeltaTalusSO_f;
		/*!
		* \brief attenuation due to receiver ground effect, in homogeneous conditions
		*/ 
		double _DeltaSolOR_h;
		/*!
		* \brief attenuation due to receiver ground effect, in favorable conditions
		*/ 
		double _DeltaSolOR_f;
		/*!
		* \brief Attenuation due to the ground effect for the SO path in homogeneous conditions
		*/ 
		double _aSolSO_h;
		/*!
		* \brief Attenuation due to the ground effect for the SO path in favorable conditions
		*/ 
		double _aSolSO_f;

		/*!
		* \brief absorption attenuation due to vertical screens
		*/ 
		double _absorptionAttenuation;
		
		/*!
		* \brief Calculates the sound absorption due to reflection
		* 
		* \param pointNMPB The reflection screen
		* 
		* \param posRef The reflection position in the elements vector
		* 
		* \param freqPos The frequency position in the spectrum array
		* 
		* \returns the calculated absorption 
		* 
		* \throws ERRScreenAbsorption
		*/
		double SoundAbsorption(ProfilePointNMPB* pointNMBP, int posRef, int freqPos);

		/*!
		* \brief Calculates data of the 2 mean planes (SO) and (OR), and embankment
		* 
		* \param withEmbankment True if embankment calculation must be done
		* 
		*/
		void MeanPlanesDataCalculation(bool withEmbankment);

		/*!
		* \brief Calculation of the ground attenuation with diffraction, source side or receiver side
		* 
		* \param aSol : ground attenuation (source side or receiver side)
		* 
		* \param deltaDif1 : diffraction attenuation during the image path (S'R or SR')
		* 
		* \param deltaDif2 : diffraction attenuation during the real path (SR)
		* 
		* \returns
		*/
		double CalculDeltaSol(double aSol, double deltaDif1, double deltaDif2);

		/*!
		 * \brief Clear data
		 */
		void ClearData()
		{
			_pathDifferenceSR_h = 0;
			_pathDifferenceSR_f = 0;
			_Adif_h = 0;
			_Adif_f = 0;
			_DeltaDifSR_h = 0;
			_DeltaDifSR_f = 0;
			_DeltaSolSO_h = 0;
			_DeltaSolSO_f = 0;
			_DeltaSolOR_h = 0;
			_DeltaSolOR_f = 0;
			_aSolSO_h = 0;
			_aSolSO_f = 0;
			_absorptionAttenuation = 0;
			_totalDiffDist = 0;
			_DeltaTalusSO_h = 0 ;
			_DeltaTalusSO_f = 0 ;
		}

	};
}

#endif